@@ -29,11 +29,18 @@
 #include <ell/ell.h>
 #include "src/dbus.h"
 #include "src/agent.h"
+#include "src/iwd.h"
 
 static unsigned int next_request_id = 0;
 
+enum agent_request_type {
+	AGENT_REQUEST_TYPE_PASSPHRASE,
+	AGENT_REQUEST_TYPE_USER_NAME_PASSWD,
+};
+
 /* Agent dbus request is done from iwd towards the agent */
 struct agent_request {
+	enum agent_request_type type;
 	struct l_dbus_message *message;
 	unsigned int id;
 	void *user_data;
@@ -157,6 +164,29 @@ done:
 	user_callback(result, passphrase, request->trigger, request->user_data);
 }
 
+static void user_name_passwd_reply(struct l_dbus_message *reply,
+					struct agent_request *request)
+{
+	const char *error, *text;
+	char *username = NULL;
+	char *passwd = NULL;
+	enum agent_result result = AGENT_RESULT_FAILED;
+	agent_request_user_name_passwd_func_t user_callback =
+		request->user_callback;
+
+	if (l_dbus_message_get_error(reply, &error, &text))
+		goto done;
+
+	if (!l_dbus_message_get_arguments(reply, "ss", &username, &passwd))
+		goto done;
+
+	result = AGENT_RESULT_OK;
+
+done:
+	user_callback(result, username, passwd,
+			request->trigger, request->user_data);
+}
+
 static void agent_finalize_pending(struct agent *agent,
 						struct l_dbus_message *reply)
 {
@@ -169,7 +199,14 @@ static void agent_finalize_pending(struct agent *agent,
 
 	pending = l_queue_pop_head(agent->requests);
 
-	passphrase_reply(reply, pending);
+	switch (pending->type) {
+	case AGENT_REQUEST_TYPE_PASSPHRASE:
+		passphrase_reply(reply, pending);
+		break;
+	case AGENT_REQUEST_TYPE_USER_NAME_PASSWD:
+		user_name_passwd_reply(reply, pending);
+		break;
+	}
 
 	if (pending->trigger) {
 		l_dbus_message_unref(pending->trigger);
@@ -227,7 +264,8 @@ static void agent_receive_reply(struct l_dbus_message *message,
 
 	agent_finalize_pending(agent, message);
 
-	agent_send_next_request(agent);
+	if (!agent->pending_id)
+		agent_send_next_request(agent);
 }
 
 static void agent_send_next_request(struct agent *agent)
@@ -255,6 +293,7 @@ static void agent_send_next_request(struct agent *agent)
 }
 
 static unsigned int agent_queue_request(struct agent *agent,
+					enum agent_request_type type,
 					struct l_dbus_message *message,
 					int timeout, void *callback,
 					struct l_dbus_message *trigger,
@@ -265,6 +304,7 @@ static unsigned int agent_queue_request(struct agent *agent,
 
 	request = l_new(struct agent_request, 1);
 
+	request->type = type;
 	request->message = message;
 	request->id = ++next_request_id;
 	request->user_data = user_data;
@@ -350,8 +390,85 @@ unsigned int agent_request_passphrase(const char *path,
 
 	l_dbus_message_set_arguments(message, "o", path);
 
-	return agent_queue_request(agent, message,
-					agent_timeout_input_request(),
+	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
+					message, agent_timeout_input_request(),
+					callback, trigger, user_data, destroy);
+}
+
+unsigned int agent_request_pkey_passphrase(const char *path,
+				agent_request_passphrase_func_t callback,
+				struct l_dbus_message *trigger,
+				void *user_data,
+				agent_request_destroy_func_t destroy)
+{
+	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
+	struct l_dbus_message *message;
+
+	if (!agent || !callback)
+		return 0;
+
+	l_debug("agent %p owner %s path %s", agent, agent->owner, agent->path);
+
+	message = l_dbus_message_new_method_call(dbus_get_bus(),
+						agent->owner, agent->path,
+						IWD_AGENT_INTERFACE,
+						"RequestPrivateKeyPassphrase");
+
+	l_dbus_message_set_arguments(message, "o", path);
+
+	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
+					message, agent_timeout_input_request(),
+					callback, trigger, user_data, destroy);
+}
+
+unsigned int agent_request_user_name_password(const char *path,
+				agent_request_user_name_passwd_func_t callback,
+				struct l_dbus_message *trigger,
+				void *user_data,
+				agent_request_destroy_func_t destroy)
+{
+	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
+	struct l_dbus_message *message;
+
+	if (!agent || !callback)
+		return 0;
+
+	l_debug("agent %p owner %s path %s", agent, agent->owner, agent->path);
+
+	message = l_dbus_message_new_method_call(dbus_get_bus(),
+						agent->owner, agent->path,
+						IWD_AGENT_INTERFACE,
+						"RequestUserNameAndPassword");
+
+	l_dbus_message_set_arguments(message, "o", path);
+
+	return agent_queue_request(agent, AGENT_REQUEST_TYPE_USER_NAME_PASSWD,
+					message, agent_timeout_input_request(),
+					callback, trigger, user_data, destroy);
+}
+
+unsigned int agent_request_user_password(const char *path, const char *user,
+				agent_request_passphrase_func_t callback,
+				struct l_dbus_message *trigger, void *user_data,
+				agent_request_destroy_func_t destroy)
+{
+	struct agent *agent = get_agent(l_dbus_message_get_sender(trigger));
+	struct l_dbus_message *message;
+
+	if (!agent || !callback)
+		return 0;
+
+	l_debug("agent %p owner %s path %s", agent, agent->owner, agent->path);
+
+	message = l_dbus_message_new_method_call(dbus_get_bus(),
+						agent->owner, agent->path,
+						IWD_AGENT_INTERFACE,
+						"RequestUserPassword");
+
+	l_dbus_message_set_arguments(message, "os", path, user ?: "");
+
+	return agent_queue_request(agent, AGENT_REQUEST_TYPE_PASSPHRASE,
+					message, agent_timeout_input_request(),
 					callback, trigger, user_data, destroy);
 }
 
@@ -381,9 +498,21 @@ bool agent_request_cancel(unsigned int req_id, int reason)
 	if (!request)
 		return false;
 
-	if (!request->message)
+	if (!request->message) {
 		send_cancel_request(agent, reason);
 
+		l_dbus_cancel(dbus_get_bus(), agent->pending_id);
+
+		agent->pending_id = 0;
+
+		if (agent->timeout) {
+			l_timeout_remove(agent->timeout);
+			agent->timeout = NULL;
+		}
+
+		agent_send_next_request(agent);
+	}
+
 	agent_request_free(request);
 
 	return true;
@@ -491,8 +620,10 @@ static bool release_agent(void *data, void *user_data)
 	return true;
 }
 
-bool agent_init(struct l_dbus *dbus)
+static int agent_init(void)
 {
+	struct l_dbus *dbus = dbus_get_bus();
+
 	agents = l_queue_new();
 
 	if (!l_dbus_register_interface(dbus, IWD_AGENT_MANAGER_INTERFACE,
@@ -500,7 +631,7 @@ bool agent_init(struct l_dbus *dbus)
 						NULL, false)) {
 		l_info("Unable to register %s interface",
 				IWD_AGENT_MANAGER_INTERFACE);
-		return false;
+		return -EIO;
 	}
 
 	if (!l_dbus_object_add_interface(dbus, IWD_AGENT_MANAGER_PATH,
@@ -509,24 +640,26 @@ bool agent_init(struct l_dbus *dbus)
 		l_info("Unable to register the agent manager object on '%s'",
 				IWD_AGENT_MANAGER_PATH);
 		l_dbus_unregister_interface(dbus, IWD_AGENT_MANAGER_INTERFACE);
-		return false;
+		return -EIO;
 	}
 
-	return true;
+	return 0;
 }
 
-bool agent_exit(struct l_dbus *dbus)
+static void agent_exit(void)
 {
+	struct l_dbus *dbus = dbus_get_bus();
+
 	l_dbus_unregister_object(dbus, IWD_AGENT_MANAGER_PATH);
 	l_dbus_unregister_interface(dbus, IWD_AGENT_MANAGER_INTERFACE);
 
 	l_queue_destroy(agents, agent_free);
 	agents = NULL;
-
-	return true;
 }
 
 void agent_shutdown(void)
 {
 	l_queue_foreach_remove(agents, release_agent, NULL);
 }
+
+IWD_MODULE(agent, agent_init, agent_exit);
