@@ -66,6 +66,7 @@ struct wiphy {
 	uint32_t feature_flags;
 	uint8_t ext_features[(NUM_NL80211_EXT_FEATURES + 7) / 8];
 	uint8_t max_num_ssids_per_scan;
+	uint32_t max_roc_duration;
 	uint16_t supported_iftypes;
 	uint16_t supported_ciphers;
 	struct scan_freq_set *supported_freqs;
@@ -355,6 +356,11 @@ uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy)
 	return wiphy->max_num_ssids_per_scan;
 }
 
+uint32_t wiphy_get_max_roc_duration(struct wiphy *wiphy)
+{
+	return wiphy->max_roc_duration;
+}
+
 bool wiphy_supports_adhoc_rsn(struct wiphy *wiphy)
 {
 	return wiphy->support_adhoc_rsn;
@@ -769,6 +775,12 @@ static void wiphy_parse_attributes(struct wiphy *wiphy,
 
 			parse_iftype_extended_capabilities(wiphy, &nested);
 			break;
+		case NL80211_ATTR_MAX_REMAIN_ON_CHANNEL_DURATION:
+			if (len != 4)
+				l_warn("Invalid MAX_ROC_DURATION attribute");
+			else
+				wiphy->max_roc_duration = *((uint32_t *) data);
+			break;
 		}
 	}
 }
@@ -1150,12 +1162,16 @@ static void setup_wiphy_interface(struct l_dbus_interface *interface)
 					NULL);
 }
 
-bool wiphy_init(struct l_genl_family *in, const char *whitelist,
-							const char *blacklist)
+static int wiphy_init(void)
 {
+	struct l_genl *genl = iwd_get_genl();
 	const struct l_settings *config = iwd_get_config();
 	const char *s = l_settings_get_value(config, "General",
 							"mac_randomize_bytes");
+	const char *whitelist = iwd_get_phy_whitelist();
+	const char *blacklist = iwd_get_phy_blacklist();
+
+	nl80211 = l_genl_family_new(genl, NL80211_GENL_NAME);
 
 	if (s && !strcmp(s, "nic"))
 		mac_randomize_bytes = 3;
@@ -1169,8 +1185,6 @@ bool wiphy_init(struct l_genl_family *in, const char *whitelist,
 		l_queue_destroy(wiphy_list, NULL);
 	}
 
-	nl80211 = in;
-
 	wiphy_list = l_queue_new();
 
 	rfkill_watch_add(wiphy_rfkill_cb, NULL);
@@ -1190,10 +1204,10 @@ bool wiphy_init(struct l_genl_family *in, const char *whitelist,
 	if (blacklist)
 		blacklist_filter = l_strsplit(blacklist, ',');
 
-	return true;
+	return 0;
 }
 
-bool wiphy_exit(void)
+static void wiphy_exit(void)
 {
 	l_strfreev(whitelist_filter);
 	l_strfreev(blacklist_filter);
@@ -1201,12 +1215,14 @@ bool wiphy_exit(void)
 	l_queue_destroy(wiphy_list, wiphy_free);
 	wiphy_list = NULL;
 
+	l_genl_family_free(nl80211);
 	nl80211 = NULL;
 	mac_randomize_bytes = 6;
 
 	l_dbus_unregister_interface(dbus_get_bus(), IWD_WIPHY_INTERFACE);
 
 	l_hwdb_unref(hwdb);
-
-	return true;
 }
+
+IWD_MODULE(wiphy, wiphy_init, wiphy_exit);
+IWD_MODULE_DEPENDS(wiphy, rfkill);
